#!/usr/bin/env bash
set -x
CENTRAL_CBOX_FILE=/config/central-box
K8S_SINGLE_MACHINE_FILE=/config/k8s-single-machine
# FIRSTUPGRADE maximum 1 match as the script runs after reboot and K8s reports v1.30.3
# before we downgrade the packeages and start upgrading versions 1 by one
FIRSTUPGRADE=`kubectl get nodes -A | grep control-plane | grep "v1.30.3" | wc -l`
isHA=`kubectl get nodes -A | grep control-plane | wc -l`

function upgrade () {
    pushd /var/kubernetes-upgrade/

    node=`hostname`
    localaddr=`cat /etc/systemd/system/kubelet.service.d/local-overrides.conf | grep -o 'node-ip[^ ]*' | cut -d= -f2- | sed 's/[^a-zA-Z0-9:.]//g'`
    if [[ $isHA -gt 1 ]]; then
        virtualaddr=`/usr/bin/jq .virtual_address /config/cluster.json | tr -d '"'`
    fi
    # v1.26.3 to v1.27.16
    dpkg -i cri-tools_1.27.1-1.1_amd64.deb
    dpkg -i kubeadm_1.27.16-1.1_amd64.deb
    if [[ $isHA -gt 1 ]]; then
        route add $virtualaddr gw $localaddr
    fi
    if [[ -f ${CENTRAL_CBOX_FILE} || -f ${K8S_SINGLE_MACHINE_FILE} ]]; then
        RC=1
        COUNTER=0
        while [ $RC -ne 0 -a $COUNTER -lt 7 ]; do
            COUNTER=$((COUNTER + 1))
            if [[ "${FIRSTUPGRADE}" == "1" ]]; then
                kubeadm upgrade apply v1.27.16 -y --force --ignore-preflight-errors=CreateJob,CoreDNSUnsupportedPlugins,CoreDNSMigration
                RC=$?
            else
                kubeadm upgrade node
                RC=0
            fi
        done
    else
        kubeadm upgrade node
    fi
    if [[ $isHA -gt 1 ]]; then
        route del $virtualaddr
        kubectl drain $node --ignore-daemonsets --delete-emptydir-data --force
    fi
    dpkg -i kubectl_1.27.16-1.1_amd64.deb
    dpkg -i kubelet_1.27.16-1.1_amd64.deb
    systemctl daemon-reload
    systemctl restart kubelet.service
    if [[ $isHA -gt 1 ]]; then
        kubectl uncordon $node
    fi
    sleep 10

    # v1.27.16 to v1.28.13
    dpkg -i cri-tools_1.28.0-1.1_amd64.deb
    dpkg -i kubeadm_1.28.13-1.1_amd64.deb
    if [[ $isHA -gt 1 ]]; then
        route add $virtualaddr gw $localaddr
    fi
    if [[ -f ${CENTRAL_CBOX_FILE} || -f ${K8S_SINGLE_MACHINE_FILE} ]]; then
        RC=1
        COUNTER=0
        while [ $RC -ne 0 -a $COUNTER -lt 7 ]; do
            COUNTER=$((COUNTER + 1))
            if [[ "${FIRSTUPGRADE}" == "1" ]]; then
                kubeadm upgrade apply v1.28.13 -y --force --ignore-preflight-errors=CreateJob,CoreDNSUnsupportedPlugins,CoreDNSMigration
                RC=$?
            else
                kubeadm upgrade node
                RC=0
            fi
        done
    else
        kubeadm upgrade node
    fi
    if [[ $isHA -gt 1 ]]; then
        route del $virtualaddr
        kubectl drain $node --ignore-daemonsets --delete-emptydir-data --force
    fi
    dpkg -i kubectl_1.28.13-1.1_amd64.deb
    dpkg -i kubelet_1.28.13-1.1_amd64.deb
    systemctl daemon-reload
    rm -f /var/lib/kubelet/pod_status_manager_state
    systemctl restart kubelet.service
    if [[ $isHA -gt 1 ]]; then
        kubectl uncordon $node
    fi
    sleep 10

    # v1.28.13 to v1.29.8
    dpkg -i cri-tools_1.29.0-1.1_amd64.deb
    dpkg -i kubeadm_1.29.8-1.1_amd64.deb
    if [[ $isHA -gt 1 ]]; then
        route add $virtualaddr gw $localaddr
    fi
    if [[ -f ${CENTRAL_CBOX_FILE} || -f ${K8S_SINGLE_MACHINE_FILE} ]]; then
        RC=1
        COUNTER=0
        while [ $RC -ne 0 -a $COUNTER -lt 7 ]; do
            COUNTER=$((COUNTER + 1))
            if [[ "${FIRSTUPGRADE}" == "1" ]]; then
                kubeadm upgrade apply v1.29.8 -y --force --ignore-preflight-errors=CreateJob,CoreDNSUnsupportedPlugins,CoreDNSMigration
                RC=$?
            else
                kubeadm upgrade node
                RC=0
            fi
        done
    else
        kubeadm upgrade node
    fi
    if [[ $isHA -gt 1 ]]; then
        route del $virtualaddr
        kubectl drain $node --ignore-daemonsets --delete-emptydir-data --force
    fi
    dpkg -i kubectl_1.29.8-1.1_amd64.deb
    dpkg -i kubelet_1.29.8-1.1_amd64.deb
    systemctl daemon-reload
    systemctl restart kubelet.service
    if [[ $isHA -gt 1 ]]; then
        kubectl uncordon $node
    fi
    sleep 10

    # v1.29.8 to 1.30.3
    dpkg -i cri-tools_1.30.1-1.1_amd64.deb
    dpkg -i kubeadm_1.30.3-1.1_amd64.deb
    if [[ $isHA -gt 1 ]]; then
        route add $virtualaddr gw $localaddr
    fi
    if [[ -f ${CENTRAL_CBOX_FILE} || -f ${K8S_SINGLE_MACHINE_FILE} ]]; then
        RC=1
        COUNTER=0
        while [ $RC -ne 0 -a $COUNTER -lt 7 ]; do
            COUNTER=$((COUNTER + 1))
            if [[ "${FIRSTUPGRADE}" == "1" ]]; then
                kubeadm upgrade apply v1.30.3 -y --force --ignore-preflight-errors=CreateJob,CoreDNSUnsupportedPlugins,CoreDNSMigration
                RC=$?
            else
                kubeadm upgrade node
                RC=0
            fi
        done
    else
        kubeadm upgrade node
    fi
    if [[ $isHA -gt 1 ]]; then
        route del $virtualaddr
        kubectl drain $node --ignore-daemonsets --delete-emptydir-data --force
    fi
    dpkg -i kubectl_1.30.3-1.1_amd64.deb
    dpkg -i kubelet_1.30.3-1.1_amd64.deb
    systemctl daemon-reload
    systemctl restart kubelet.service
    if [[ $isHA -gt 1 ]]; then
        kubectl uncordon $node
    fi

    popd
    rm -rf /var/kubernetes-upgrade
    pushd /kubernetes/images
    rm -f kube-controller-manager_v1.27.16.txz kube-controller-manager_v1.28.13.txz kube-controller-manager_v1.29.8.txz
    rm -f kube-proxy_v1.27.16.txz  kube-proxy_v1.28.13.txz kube-proxy_v1.29.8.txz kube-scheduler_v1.27.16.txz
    rm -f kube-scheduler_v1.28.13.txz kube-scheduler_v1.29.8.txz coredns_v1.10.1.txz
    rm -f kube-apiserver_v1.27.16.txz kube-apiserver_v1.28.13.txz kube-apiserver_v1.29.8.txz
    popd
    rm -f /config/.upgrade_kubernetes2
}

# Wait cluster to go in running state
# This function will sleep ${SLEEP} seconds before ends
SLEEP=20
timer=1
function wait_cluster () {
  sleep 10
  while [ $timer -le ${SLEEP} ]; do
    if [[ "active" == "$(systemctl is-active kubelet.service)" ]]; then
      return # proceed with upgrade
    fi
    sleep 30
    timer=$((timer + 1))
  done
  exit 0 # timeouts - SLEEP * 30 - 20*30=600s (10 min)
}

wait_cluster
upgrade
